/*
    InMoDe is a collection of seven tools for learning, leveraging, and visualizing intra-motif dependencies within DNA binding sites and similar functional nucleotide sequences. 
    More information can be found at <http://jstacs.de/index.php/InMoDe>
    Copyright (C) 2017  Ralf Eggeling, Ivo Grosse, and Jan Grau
    
    InMoDe is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    InMoDe is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, see <http://www.gnu.org/licenses>.
*/
package de.jstacs.projects.inmode.models.latentVariables;

import de.jstacs.data.AlphabetContainer;
import de.jstacs.io.NonParsableException;
import de.jstacs.sequenceScores.statisticalModels.trainable.AbstractTrainableStatisticalModel;


/**
 * Abstract class for models that require latent variables, such as mixture models and promoter models and are to be learned via a stochastic (EM) algorithm.
 * @author Ralf Eggeling
 * 
 */

public abstract class AbstractLatentVariableModel extends AbstractTrainableStatisticalModel{

	protected static final String XML_TAG = "AbstractLatentVariableModel";
		
	protected int numberOfInitialIterations;
	
	protected int numberOfAdditionalIterations;
	
	protected int numberOfRestarts;
	
	/**
	 * This is the standard constructor.
	 * @param alphabets The alphabet container.
	 * @param length The length of the input sequences.	 
	 */
	public AbstractLatentVariableModel(AlphabetContainer alphabets, int length) {
		super(alphabets, length);
	}

	/**
	 * The standard constructor for the interface {@link de.jstacs.Storable}. Creates a new {@link AbstractLatentVariableModel} out of a {@link StringBuffer}.
	 * @param sb the {@link StringBuffer} to be parsed
	 * @throws NonParsableException  if the {@link AbstractLatentVariableModel} could not be reconstructed out of
	 *             the {@link StringBuffer} <code>sb</code>      
	 */
	public AbstractLatentVariableModel(StringBuffer sb) throws NonParsableException {
		super(sb);
	}
	
}
