/*
    InMoDe is a collection of seven tools for learning, leveraging, and visualizing intra-motif dependencies within DNA binding sites and similar functional nucleotide sequences. 
    More information can be found at <http://jstacs.de/index.php/InMoDe>
    Copyright (C) 2017  Ralf Eggeling, Ivo Grosse, and Jan Grau
    
    InMoDe is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    InMoDe is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, see <http://www.gnu.org/licenses>.
*/
package de.jstacs.projects.inmode.models.variableStructure;

import de.jstacs.data.AlphabetContainer;
import de.jstacs.io.NonParsableException;
import de.jstacs.io.XMLParser;
import de.jstacs.projects.inmode.models.latentVariables.AbstractLatentVariableModel;
import de.jstacs.sequenceScores.statisticalModels.trainable.AbstractTrainableStatisticalModel;


/**
 * Abstract class for all trainable statistical models that include a model (structure) selection. 
 * @author Ralf Eggeling
 *
 */

public abstract class AbstractVariableStructureModel extends AbstractTrainableStatisticalModel {
	
	protected boolean trained;
	
	/** 
	 * This is the standard constructor.
	 * @param alphabets used alphabet container.
	 * @param length the length of the sequences to be handled (0 indicates arbitrary length).
	 */
	public AbstractVariableStructureModel(AlphabetContainer alphabets,
			int length) {
		super(alphabets, length);
	}
	
	/**
	 * The standard constructor for the interface {@link de.jstacs.Storable}. Creates a new {@link AbstractVariableStructureModel} out of a {@link StringBuffer}.
	 * @param sb the {@link StringBuffer} to be parsed
	 * @throws NonParsableException  if the {@link AbstractVariableStructureModel} could not be reconstructed out of
	 *             the {@link StringBuffer} <code>sb</code>      
	 */
	public AbstractVariableStructureModel(StringBuffer sb) throws NonParsableException {
		super(sb);
	}
	
	public boolean isInitialized() {
		return trained;
	}
	
	public AbstractVariableStructureModel clone() throws CloneNotSupportedException {
		return (AbstractVariableStructureModel) super.clone();
	}
	
	protected abstract void fromXML(StringBuffer xml) throws NonParsableException;
	/**
	 * The function returns the score of the currently selected structure. It should have an interpretation in log scale, i.e., the sum of two scores are supposed to be a meaningful quantity.
	 * @return score
	 */
	public abstract double getLogScoreOfCurrentStructure();
	
}
