/*
    InMoDe is a collection of seven tools for learning, leveraging, and visualizing intra-motif dependencies within DNA binding sites and similar functional nucleotide sequences. 
    More information can be found at <http://jstacs.de/index.php/InMoDe>
    Copyright (C) 2017  Ralf Eggeling, Ivo Grosse, and Jan Grau
    
    InMoDe is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    InMoDe is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, see <http://www.gnu.org/licenses>.
*/
package de.jstacs.projects.inmode.utils;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Rectangle2D;
import de.jstacs.utils.SeqLogoPlotter;
import de.jstacs.utils.ToolBox;

/**
 * Extention of {@link SeqLogoPlotter} that allows customized plotting of individual plot elements for use in {@link CSLPlotter}.
 * @author Ralf Eggeling
 *
 */
public class CustomizedSeqLogoPlotter extends SeqLogoPlotter {
	
	static Color[][] colors = new Color[][]{
		{Color.GREEN,Color.BLUE,Color.ORANGE,Color.RED},
		{Color.decode("#15A456"),Color.decode("#3A57A1"),Color.decode("#FEBE3C"),Color.decode("#ED3A42")},
		{Color.decode("#4DAF4A"),Color.decode("#377EB8"),Color.decode("#FF7F00"),Color.decode("#E41A1C")},
		{Color.LIGHT_GRAY,Color.DARK_GRAY,Color.GRAY,Color.BLACK}
			
			
	};
	
	protected static void plotStack(Graphics2D g, int x, int y, int w, int h, double[] p, String label, boolean scaled, int border, int colorscheme) {
		plotLogo(g, x+border, y-25, w-2*border, h-25,  p, scaled,colorscheme);
		g.setColor( Color.BLACK );
	//	g.drawLine(x+border, y-20, x+w-border, y-20);
		g.drawString(label, (int) (x+Math.round(w-g.getFontMetrics().getStringBounds( label, g ).getWidth())/2), y-5);
	}
	protected static void plotYAxis(Graphics2D g, int x, int y, int h) {
		String[] labs = {"0", "0.5", "1", "1.5", "2"};

		double wl = h*0.4;
		double x2 = wl*0.9;
		double h2 = h-25;
		double y2 = y-25;
		g.drawLine( x+(int)(x2*0.94), (int)y2, x+(int)(x2*0.94), (int)(y2-h2) );
		for(int i=0;i<=4;i++){
			g.drawLine( x+(int)(x2*0.7), (int)(y2-i*h2/4.0), x+(int)(x2*0.94), (int)(y2-i*h2/4.0) );
			Rectangle2D rect = g.getFontMetrics().getStringBounds( labs[i], g );
			g.drawString( labs[i], x+(int)(x2*0.6-rect.getWidth()), (int)(y2-i*h2/4.0 - rect.getCenterY()) );
		}
		AffineTransform back = g.getTransform();
		g.rotate( -Math.PI/2 );
		String labY = "bits";
		Rectangle2D rect = g.getFontMetrics().getStringBounds( labY, g );

		g.drawString(labY,-(int)(y2-2*h2/4.0 + rect.getCenterX()), (int)(x+rect.getWidth()/2d));
		g.setTransform( back );
	}
	
	
	protected static void plotLogo(Graphics2D g, double x, double y, double w, double h, double[] p, boolean scaled, int colorscheme){
		
		//y += h;
	//	colorscheme=2;
		
		double ic = 1; 
		if(scaled)
			ic=getICScale(p);
		h *= ic;
		//System.out.println("h: "+h);
		
		double[] mp = p.clone();
		for(int i=0;i<mp.length;i++){
			mp[i] *= -1;
		}
		
		int[] r = ToolBox.rank( mp, false );
		int[] order = new int[r.length];
		for(int i=0;i<r.length;i++){
			order[r[i]] = i;
		}
		
		
		for(int i=0;i<order.length;i++){
			double curr = p[order[i]];
			if(order[i] == 0){
				g.setColor( colors[colorscheme][0] );
				g.fill(getA( x, y, w, h*curr ));
			}else if(order[i] == 1){
				g.setColor( colors[colorscheme][1]);
				g.fill(getC( x, y, w, h*curr ));
			}else if(order[i] == 2){
				g.setColor( colors[colorscheme][2] );
				g.fill(getG( x, y, w, h*curr ));
			}else{
				g.setColor( colors[colorscheme][3] );
				g.fill(getT( x, y, w, h*curr ));
			}
			//System.out.println("y: "+y);
			y -= h*curr;
		}
		//System.out.println(y);
	}
	private static Area getC( double x, double y, double w, double h ){
		
		Shape s = new Ellipse2D.Double( 0, -90, 90, 90 );
		Area a1 = new Area( s );
		
		Shape s2 = new Ellipse2D.Double( 15, -75, 60, 60 );
		Area a2 = new Area(s2);
		
		a1.subtract( a2 );
		
		Shape s3 = new Rectangle2D.Double( 65, -60, 30, 30 );
		Area a3 = new Area( s3 );
		
		a1.subtract( a3 );
		
		AffineTransform t = new AffineTransform();
		t.scale( 1d/88d, 1d/90d );
		t.scale( w, h );
		a1.transform( t );
		t = new AffineTransform();
		t.translate( x, y );
		a1.transform( t );
		return a1;
	}
	
	private static Area getT( double x, double y, double w, double h ){
		
		Shape s = new Rectangle2D.Double( 37.5, -100, 15, 100 );
		Area a1 = new Area( s );
		
		Shape s2 = new Rectangle2D.Double(0,-100,90,15);
		Area a2 = new Area( s2 );
		
		a1.add( a2 );
		
		AffineTransform t = new AffineTransform();
		t.scale( 1d/90d, 1d/100d );
		t.scale( w, h );
		a1.transform( t );
		t = new AffineTransform();
		t.translate( x, y );
		a1.transform( t );
		return a1;
		
	}
	
	private static Area getG( double x, double y, double w, double h ){
		
		Shape s = new Ellipse2D.Double( 0, -90, 90, 90 );
		Area a1 = new Area( s );
		
		Shape s2 = new Ellipse2D.Double( 15, -75, 60, 60 );
		Area a2 = new Area(s2);
		
		a1.subtract( a2 );
		
		Shape s3 = new Rectangle2D.Double( 65, -60, 30, 30 );
		Area a3 = new Area( s3 );
		
		a1.subtract( a3 );
		
		Shape s4 = new Rectangle2D.Double( 55, -40, 35, 15 );
		Area a4 = new Area( s4 );
		
		a1.add( a4 );
		
		Shape s5 = new Rectangle2D.Double( 80, -40, 10, 40 );
		Area a5 = new Area( s5 );
		
		a1.add( a5 );
		
		AffineTransform t = new AffineTransform();
		t.scale( 1d/90d, 1d/90d );
		t.scale( w, h );
		a1.transform( t );
		t = new AffineTransform();
		t.translate( x, y );
		a1.transform( t );
		return a1;
	}
	
	private static Area getA( double x, double y, double w, double h ){
		
		Shape s = new Polygon( new int[]{0,40,50,90,75,45,45,15,0}, new int[]{0,-100,-100,0,0,-80,-80,0,0}, 9 );
		Area a = new Area( s );
		
		Shape s2 = new Polygon(new int[]{20,70,70,20},new int[]{-35,-35,-50,-50},4);
		Area a2 = new Area( s2 );
		
		a.add( a2 );
		
		AffineTransform t = new AffineTransform();
		t.scale( 1.0/90.0, 1.0/100.0 );
		t.scale( w, h );
		a.transform( t );
		t = new AffineTransform();
		t.translate( x, y );
		a.transform( t );
		return a;
	}
	
	
	
}
